# -*- coding: utf-8 -*-

#############################################################################
# Copyright (c) 2014-2018 Redfern Electronics Limited. All rights reserved. #
#############################################################################

import collections
import os
import sys
import canvas
import statements
import blocks
import toolbox
import crumble_device
import compile
import vm
import wx
import datetime
try:
    from pubsub import pub
except ImportError:
    if wx.VERSION[0] < 3:
        from wx.lib.pubsub import setupkwargs
    from wx.lib.pubsub import pub
import xml.etree.cElementTree as ET
import utils
import struct
import urllib.request, urllib.error, urllib.parse
import lang
from lang import _

version = '1.5.8'
vm_version = 0x1B # Set to zero to force VM update
#vm_version = 0

class CrumbleFrame(wx.Frame):
    def __init__(self):
        super(CrumbleFrame, self).__init__(None, wx.ID_ANY, 'Crumble Version %s' % version, size=(800,600))

        self.filename = ''

        script_path = os.path.dirname(os.path.realpath(sys.argv[0]))

        if os.path.exists(os.path.join(script_path, 'crumble.exe')):
            favicon = wx.Icon('crumble.exe', wx.BITMAP_TYPE_ICO, 16, 16)
            self.SetIcon(favicon)

        self.log_path = ""
        self.logging = False
        if os.path.exists(os.path.join(script_path, 'crumblelogpath.txt')):
            log_path_file  = open(os.path.join(script_path, 'crumblelogpath.txt'))
            self.log_path = log_path_file.readline()
            log_path_file.close()
            if os.path.isdir(self.log_path):
                self.logging = True

        self.SetBackgroundColour((0xff,0xff,0xff))

        self.menus = []

        menu_bar = wx.MenuBar()

        file_menu = wx.Menu()
        file_menu.Append(wx.ID_NEW, '&New\tCtrl+N')
        file_menu.Append(wx.ID_OPEN, '&Open\tCtrl+O')
        file_menu.Append(wx.ID_SAVE, '&Save\tCtrl+S')
        file_menu.Append(wx.ID_SAVEAS, 'Save &As')
        file_menu.AppendSeparator()

        self.Bind(wx.EVT_MENU, self.on_new, id=wx.ID_NEW)
        self.Bind(wx.EVT_MENU, self.on_open, id=wx.ID_OPEN)
        self.Bind(wx.EVT_MENU, self.on_save, id=wx.ID_SAVE)
        self.Bind(wx.EVT_MENU, self.on_save_as, id=wx.ID_SAVEAS)

        exit_menu_item = wx.MenuItem(file_menu, wx.ID_EXIT, '&Exit')
        file_menu.Append(exit_menu_item)
        self.Bind(wx.EVT_MENU, self.on_exit, exit_menu_item)
        self.Bind(wx.EVT_CLOSE, self.on_exit)

        program_menu = wx.Menu()
        send_program_menu_item = program_menu.Append(wx.ID_ANY, 'Program Crumble and &Run\tCtrl+R')
        stop_menu_item = program_menu.Append(wx.ID_ANY, 'Stop')
        self.Bind(wx.EVT_MENU, self.on_send_program, send_program_menu_item)
        self.Bind(wx.EVT_MENU, self.on_stop, stop_menu_item)

        view_menu = wx.Menu()
        zoom_in_menu_item = view_menu.Append(wx.ID_ANY, 'Zoom In\tCtrl++')
        zoom_out_menu_item = view_menu.Append(wx.ID_ANY, 'Zoom Out\tCtrl+-')
        actual_size_menu_item = view_menu.Append(wx.ID_ANY, 'Actual Size\tCtrl+0')
        self.Bind(wx.EVT_MENU, self.on_zoom_in, zoom_in_menu_item)
        self.Bind(wx.EVT_MENU, self.on_zoom_out, zoom_out_menu_item)
        self.Bind(wx.EVT_MENU, self.on_actual_size, actual_size_menu_item)

        # language_menu = wx.Menu()
        # self.language_ids = {}
        # for language in sorted(lang.LANGUAGES.iterkeys()):
        #     menu_item = language_menu.Append(wx.ID_ANY, language)
        #     self.Bind(wx.EVT_MENU, self.on_language, menu_item)
        #     self.language_ids[menu_item.GetId()] = language

        # crumble_inside_menu = wx.Menu()
        # crumble_by_redfern = crumble_inside_menu.Append(wx.ID_ANY, 'Crumble by Redfern (default)',
        #                                                 'Crumble by Redfern (default)', kind=wx.ITEM_RADIO)
        # stitch_up_by_redfern = crumble_inside_menu.Append(wx.ID_ANY, 'StitchUp by Redfern',
        #                                                   'Stitch-Up by Redfern', kind=wx.ITEM_RADIO)
        # logjam_up_by_redfern = crumble_inside_menu.Append(wx.ID_ANY, 'LogJam by Redfern',
        #                                                   'Stitch-Up by Redfern', kind=wx.ITEM_RADIO)
        # # crumble_inside_menu.AppendSeparator()
        # zippy_by_companya = crumble_inside_menu.Append(wx.ID_ANY, 'Zippy by Company A',
        #                                                 'Zippy by Company A', kind=wx.ITEM_RADIO)
        # robofish_by_companya = crumble_inside_menu.Append(wx.ID_ANY, 'Robofish by Company A',
        #                                                 'Robofish by Company A', kind=wx.ITEM_RADIO)
        # # crumble_inside_menu.AppendSeparator()
        # twinkle_tree_by_companyb = crumble_inside_menu.Append(wx.ID_ANY, 'TwinkleTree by Company B',
        #                                                 'TwinkleTree by Company B', kind=wx.ITEM_RADIO)
        # Junior_by_companyb = crumble_inside_menu.Append(wx.ID_ANY, 'Junior by Company B',
        #                                                   'Junior by Company B', kind=wx.ITEM_RADIO)

        menu_bar.Append(file_menu, '&File')
        menu_bar.Append(program_menu, '&Program')
        menu_bar.Append(view_menu, '&View')
        # menu_bar.Append(language_menu, '&Language')
        # menu_bar.Append(crumble_inside_menu, '&Crumble Inside')
        self.SetMenuBar(menu_bar)

        toolbar = self.CreateToolBar()
        toolbar.SetToolBitmapSize((32, 32))
        run_tool = toolbar.AddTool(wx.ID_ANY, 'Run', wx.Bitmap(utils.get_path() + '/PNG/Run tool.png'))
        stop_tool = toolbar.AddTool(wx.ID_ANY, 'Stop', wx.Bitmap(utils.get_path() + '/PNG/Stop tool.png'))
        undelete_tool = toolbar.AddTool(wx.ID_ANY, 'Undelete', wx.Bitmap(utils.get_path() + '/PNG/Undelete.png'))
        
        self.status = wx.StaticText(toolbar, wx.ID_ANY, _('Ready'), size=(320,-1))
        if lang.LANGUAGE == 'chinese':
            self.status.SetFont(wx.Font(16, wx.DEFAULT, wx.NORMAL, wx.NORMAL, faceName='Microsoft YaHei'))
        toolbar.AddControl(self.status)

        toolbar.AddStretchableSpace()
        zoom_out_tool = toolbar.AddTool(wx.ID_ANY, 'Zoon Out', wx.Bitmap(utils.get_path() + '/PNG/Zoom out.png'))
        zoom_in_tool = toolbar.AddTool(wx.ID_ANY, 'Zoon In', wx.Bitmap(utils.get_path() + '/PNG/Zoom in.png'))

        toolbar.Realize()

        self.Bind(wx.EVT_TOOL, self.on_send_program, run_tool)
        self.Bind(wx.EVT_TOOL, self.on_stop, stop_tool)
        self.Bind(wx.EVT_TOOL, self.on_undelete, undelete_tool)
        self.Bind(wx.EVT_TOOL, self.on_zoom_out, zoom_out_tool)
        self.Bind(wx.EVT_TOOL, self.on_zoom_in, zoom_in_tool)

        self.canvas = canvas.Canvas(self, wx.ID_ANY)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.canvas, 1, flag=wx.EXPAND)
        self.SetSizer(sizer)

        self.canvas.add(toolbox.ToolboxBlock(0, 0), self.canvas.toolboxes)

        self.monitor_timer = wx.Timer(self, wx.ID_ANY)
        self.Bind(wx.EVT_TIMER, self.on_monitor_timer, self.monitor_timer)

        self.check_for_updates()

        # self.canvas.add(statements.FunctionBlock(0,0))

        self.Show()

    @property
    def filename(self):
        return self._filename

    @filename.setter
    def filename(self, new_name):
        self._filename = new_name
        if self._filename:
            self.Title = 'Crumble Version %s - %s' % (version, os.path.basename(self._filename))
        else:
            self.Title = 'Crumble Version %s' % version

    def check_for_updates(self):
        try:
            file = urllib.request.urlopen('http://www.redfernelectronics.co.uk/crumble_version.txt')
            online_version = file.read()
        except: # ValueError:     # Catch all errors :( Firewalls and proxies seem to cause lots of different errors
            pass
        else:
            try:
                (online_major, online_minor, online_patch) = [int(x) for x in online_version.split('.')]
            except ValueError:
                pass
            else:
                (major, minor, patch) = [int(x) for x in version.split('.')]
                update_available = False
                if online_major > major:
                    update_available = True
                elif online_major == major and online_minor > minor:
                    update_available = True
                elif online_major == major and online_minor == minor and online_patch > patch:
                    update_available = True
                if update_available:
                    self.status.Label = 'New software available (%s)' % online_version

    def on_exit(self, event):
        if self.canvas.modified:
            result = wx.MessageBox(_("Do you want to save changes?"), "Crumble",
                             wx.ICON_QUESTION | wx.YES_NO | wx.CANCEL, self)
            if result == wx.YES:
                if not self.save():
                    return
            elif result == wx.CANCEL:
                return

        self.Destroy()

    def write_log(self):
        canvas_element = self.canvas_to_xml()
        root = ET.Element('CrumbleDocument', {'version': version})
        vars = self.canvas.toolboxes[0].variables_button.get_vars()
        vars_element = ET.Element('Variables')
        for var in vars:
            var_element = ET.Element('Variable')
            var_element.text = var
            vars_element.append(var_element)
        root.append(vars_element)
        root.append(canvas_element)
        tree = ET.ElementTree(root)
        timestamp = datetime.datetime.now().strftime("%Y %B %d %H-%M-%S")
        tree.write(os.path.join(self.log_path, 'Crumble Log ' + timestamp+ '.xml'))

    def on_language(self, event):
        id_selected = event.GetId()
        language = lang.LANGUAGES[self.language_ids[id_selected]]
        self.set_language(language)

    def on_send_program(self, event):
        self.status.Label = 'Programming Crumble...'
        self.status.Refresh()
        self.status.Update()
        wx.GetApp().Yield()

        self.monitor_stop() # Stop monitoring (also updates graphics e.g. variable monitors to '?' in case var not present in new prog)
        
        try:
            crumble_device.init_vm() # Stop current program (also turns off motor outputs etc)
        except IOError:
            pass # Continue to compile
        
        if self.logging == True:
            self.write_log()

        try:
            start_block = None
            for child in self.canvas.children:
                if type(child) == statements.WhenRunBlock:
                    start_block = child

            basic_code = ''
            inits = collections.OrderedDict()
            setups = collections.OrderedDict()

            if start_block:
                blocks = self.canvas.find_all_attached(start_block)
                for block in blocks[1:]:
                    basic_code += block.basic_code(inits, setups)

            init_code = ''
            for init in inits.keys():
                init_code += init

            setup_code = ''
            for setup in setups.keys():
                setup_code += setup

            basic_code = init_code + '\n' + basic_code

            # print('Code:')
            # print(basic_code)

            machine_code = compile.compile(basic_code)
        except:
            self.status.Label = _('ERROR: Compiler problem')
            return

        # for i in range(0, len(machine_code), 8):
        #     remaining = len(machine_code) - i
        #     if remaining >= 8:
        #         print '0x%04x: ' % i,
        #         print '0x%04x 0x%04x 0x%04x 0x%04x 0x%04x 0x%04x 0x%04x 0x%04x' % tuple(machine_code[i:i+8])
        #     else:
        #         print '0x%04x: ' % i,
        #         for j in range(i, i + remaining):
        #             print '0x%04x ' % machine_code[j],

        percentage = (len(machine_code) * 100 ) // crumble_device.BYTE_CODE_LENGTH
        # print(percentage)
        
        if len(machine_code) > crumble_device.BYTE_CODE_LENGTH:
            self.status.Label = _('ERROR: Program too large')
            return

        try:
            loaded_vm_version = crumble_device.read_vm_version()
            crumble_device.erase_vm_first_page()
            crumble_device.write_byte_code(machine_code)
            if vm_version==0 or loaded_vm_version!=vm_version:
                crumble_device.program_hex(vm.hex_data)
            else:
                crumble_device.program_hex(vm.hex_data, first_page_only=True)
            crumble_device.init_vm()
            crumble_device.send_vm_command(crumble_device.VM_START)
        except IOError:
            self.status.Label = _('ERROR: Crumble not found!')
        else:
            if percentage < 1.0:
                self.status.Label = _('Programming successful ') + '(' + _('less than 1% memory used') + ')'
            else:
                self.status.Label = _('Programming successful ') + '(' + str(percentage) + '% ' + _('memory used') + ')'
            self.monitor_start()

    def on_stop(self, event):
        try:
            crumble_device.init_vm()
            # crumble_device.stop_vm()
        except IOError:
            self.status.Label = _('ERROR: Crumble not found!')
        else:
            self.status.Label = _('Stopped')

    def on_undelete(self, event):
        if self.canvas.undelete():
            self.status.Label = _('Undeleted blocks')
        else:
            self.status.Label = _('Nothing to undelete')

    def on_new(self, event):
        if self.canvas.modified:
            result = wx.MessageBox(_("Do you want to save changes?"), "Crumble",
                             wx.ICON_QUESTION | wx.YES_NO | wx.CANCEL, self)
            if result == wx.YES:
                if not self.save():
                    return
            elif result == wx.CANCEL:
                return

        self.canvas.toolboxes[0].variables_button.reset()
        self.canvas.toolboxes[0].basic_button.load_tools()

        self.canvas.delete_all()
        self.filename = ''
        self.canvas.modified = False

    def on_save(self, event):
        self.save()

    def on_save_as(self,event):
        self.save(save_as=True)

    def on_open(self, event):
        self.open('')

    def on_zoom_in(self, event):
        if self.canvas.zoom < 2.0:
            self.canvas.zoom += 0.25

    def on_zoom_out(self, event):
        if self.canvas.zoom > 1.0:
            self.canvas.zoom -= 0.25

    def on_actual_size(self, event):
        self.canvas.zoom = 1.0

    def open(self, filename):
        if self.canvas.modified:
            result = wx.MessageBox(_("Do you want to save changes?"), "Crumble",
                             wx.ICON_QUESTION | wx.YES_NO | wx.CANCEL, self)
            if result == wx.YES:
                if not self.save():
                    return
            elif result == wx.CANCEL:
                return

        if not filename:
            open_dialog = wx.FileDialog(self, "Open Crumble file", "", "",
                                        "Crumble files (*.crm)|*.crm", wx.FD_OPEN | wx.FD_FILE_MUST_EXIST)
            if open_dialog.ShowModal() == wx.ID_CANCEL:
                return
            self.filename = open_dialog.GetPath()
        else:
            self.filename = filename

        self.canvas.delete_all()
        self.canvas.modified = False
        try:
            tree = ET.ElementTree(file=self.filename)
            root = tree.getroot()
            file_version = root.attrib['version']
            if 'language' in root.attrib:
                self.set_language(root.attrib['language'])
            vars_element = root.find('Variables')
            if vars_element:
                vars = []
                for var_element in vars_element:
                    vars.append(var_element.text)
                self.canvas.toolboxes[0].variables_button.set_vars(vars)
            else:
                self.canvas.toolboxes[0].variables_button.reset()
            self.canvas.toolboxes[0].basic_button.load_tools()
            self.canvas_from_xml(root.find('Canvas'))
        except ET.ParseError:
            wx.MessageBox("Could not open: file appears to be corrupted :-(", "Crumble", wx.ICON_ERROR | wx.OK, self)
        except ValueError:
            wx.MessageBox("Could not open: file saved in version %s" % file_version, "Crumble", wx.ICON_ERROR | wx.OK, self)

    def save(self, save_as=False):
        if not self.filename or save_as:
            save_dialog = wx.FileDialog(self, "Save Crumble file", "", "",
                                        "Crumble files (*.crm)|*.crm", wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT)
            if save_dialog.ShowModal() == wx.ID_CANCEL:
                return False
            else:
                self.filename = save_dialog.GetPath()
                if self.filename[-4:] != '.crm':
                    self.filename += '.crm'
        canvas_element = self.canvas_to_xml()
        root = ET.Element('CrumbleDocument', {'version': version, 'language': lang.LANGUAGE})
        vars = self.canvas.toolboxes[0].variables_button.get_vars()
        vars_element = ET.Element('Variables')
        for var in vars:
            var_element = ET.Element('Variable')
            var_element.text = var
            vars_element.append(var_element)
        root.append(vars_element)
        root.append(canvas_element)
        tree = ET.ElementTree(root)
        tree.write(self.filename)
        self.canvas.modified = False
        return True

    def set_language(self, language):
        if language in lang.LANGUAGES.values():
            root = self.canvas_to_xml()
            lang.LANGUAGE = language
            self.canvas.delete_all()
            self.canvas_from_xml(root)
            self.canvas.remove(self.canvas.toolboxes[0], self.canvas.toolboxes)
            self.canvas.add(toolbox.ToolboxBlock(0, 0), self.canvas.toolboxes)

    def canvas_to_xml(self):
        root = ET.Element('Canvas')
        if self.canvas.children:
            for child in self.canvas.children:
                if not child.attached_to and hasattr(child, 'to_xml'):
                    root.append(child.to_xml(position=True))
        return root

    def canvas_from_xml(self, canvas_element):
        for element in canvas_element:
            blocks.from_xml(element, self.canvas)

    def monitor_start(self):
        self.monitor_timer.Start(250)

    def monitor_stop(self):
        pub.sendMessage('crumble.monitor.stopped')
        self.monitor_timer.Stop()

    def on_monitor_timer(self, event):
        try:
            data = crumble_device.read_ram(102, 64)[2:] # 104 is address of stack space, but can't read first byte
        except IOError:
            self.monitor_stop()
            self.status.Label = 'Crumble disconnected'
        else:
            if data:    # If read times out, but no IO Error, data is empty - don't process (seems to happen on Pi when CPU loaded)
                i = 0
                values = {}
                for var_name in compile.global_vars:
                    value = struct.unpack('<h', bytearray(data[i:i+2]))[0]
                    values[var_name.replace('_', ' ')] = value
                    i+=2
                    if i >= len(data):
                        break
                pub.sendMessage('crumble.variable.values', values=values)


class CrumbleApp(wx.App):
    def __init__(self, *args, **kwargs):
        wx.App.__init__(self, *args, **kwargs)

        # This catches events when the app is asked to activate by some other
        # process
        self.Bind(wx.EVT_ACTIVATE_APP, self.OnActivate)

    def OnInit(self):
        self.frame = CrumbleFrame()
        self.frame.Show()

        if wx.Platform != '__WXMAC__':
            if len(sys.argv) == 2:
                self.frame.open(sys.argv[1])

        return True

    def BringWindowToFront(self):
        try: # it's possible for this event to come when the frame is closed
            self.GetTopWindow().Raise()
        except:
            pass

    def OnActivate(self, event):
        # if this is an activate event, rather than something else, like iconize.
        if event.GetActive():
            self.BringWindowToFront()
        event.Skip()

    def MacOpenFile(self, filename):
        pass

    def MacOpenFiles(self, filenames):
        """Called for files droped on dock icon, or opened via finders context menu"""
        self.frame.open(filenames[0])

    def MacReopenApp(self):
        """Called when the doc icon is clicked, and ???"""
        self.BringWindowToFront()

    def MacNewFile(self):
        pass

    def MacPrintFile(self, file_path):
        pass


if __name__=='__main__':
    if len(sys.argv) > 1 and sys.argv[1]=='-v':
        print(version, end='')
        exit()

    app = CrumbleApp(False)
    # app = CrumbleApp(True) # Use this to see errors and print statements

    app.MainLoop()
